require 'bundler'
Bundler.require
require "spec_helper"
require 'rspec'
require 'rspec/mocks'
describe CucumberTestrail do 
  before(:each) do
  ENV['TESTRAIL_BASE_URL']='https://paulokikiade.testrail.com' 
  ENV['TESTRAIL_USER']    ='test.rail@bbc.com'
  ENV['TESTRAIL_PASSWORD']='a_password'
  end

  let(:world)  do
    world  = class_double('Funky')
    world.extend subject 
    world
  end
  
  describe "#testrail" do
   it "should raise an exception if any of the environment variables are not set up" do
    ['TESTRAIL_BASE_URL','TESTRAIL_USER','TESTRAIL_PASSWORD'].each do | var |
      save_env = ENV[var]
      begin
        ENV[var] = nil
        expect{world.testrail}.to raise_exception(RuntimeError,/#{var}/)
      ensure
        ENV[var] = save_env
      end
    end  
   end
  end

  describe "#send result" do 
    it 'should not send the result if scenario.ignore_testrail is true' do
      scenario = instance_double('Scenario',ignore_testrail?: true)
      expect(world.send_result(scenario)).to be_falsey
    end  
  end

  describe "#locate_a_test_case_by_name" do
    it "doesn't find a phoney test" do
      VCR.use_cassette('title_not_found') do
        scenario = instance_double("Scenario",title:'A silly title',project:1,suite:1,sub_section:2)
        expect(world.locate_a_test_case_by_name(scenario)).to be_nil
      end
    end

    it "does find a real test" do
      VCR.use_cassette('title_is_found') do
        scenario = instance_double("Scenario",title:'Carousel - Test Case 1',project:1,suite:1,sub_section:2)
        expect(world.locate_a_test_case_by_name(scenario)).not_to be_nil
      end
    end
  end

  describe '#create_a_test_case_from_scenario' do
    let(:testrail_testcase) {{'title'=>'A test scenario','type_id'=>1,'custom_steps'=>'Given some steps'}}
    let(:scenario) {instance_double("Scenario",suite:2,sub_section:2,testrail_testcase:testrail_testcase)}
    it "should get a testcase id" do
      VCR.use_cassette('create_a_test_case') do
        expect(world.create_a_test_case_from_scenario(scenario)['id']).not_to be_nil
      end
    end

    it "should put the steps into custom steps" do
      VCR.use_cassette('create_a_test_case') do
        expect(world.create_a_test_case_from_scenario(scenario)['custom_steps']).to eq('Given some steps')
      end
    end
    it "should add the test case to the test suite" do
      VCR.use_cassette('create_a_test_case') do
        expect(world.create_a_test_case_from_scenario(scenario)['section_id']).to eq(2)
      end
    end
    it "should set the testcase title" do
      VCR.use_cassette('create_a_test_case') do
        expect(world.create_a_test_case_from_scenario(scenario)['title']).to eq('A test scenario')
      end
    end  
  end  

  describe '#send_test_result_to_testrail' do 
    let(:testrail_test_report) {   {status_id:1,defects:'none',comment:'comment'}}
    let(:scenario) {instance_double("Scenario",testcase:10,testrun:5,testrail_test_report:testrail_test_report)}
    it 'should return a valid test result' do 
      VCR.use_cassette('create_a_test_result') do
        expect(world.send_test_result_to_testrail(scenario)['id']).not_to be_nil
      end
    end
    
    it 'should set the defects' do 
      VCR.use_cassette('create_a_test_result') do
        expect(world.send_test_result_to_testrail(scenario)['defects']).to eq('none')
      end
    end

    it 'should set the comment' do 
      VCR.use_cassette('create_a_test_result') do
        expect(world.send_test_result_to_testrail(scenario)['comment']).to eq('comment')
      end
    end
  end
end
